﻿#script "CS-PS4CropPicture_GUI"
#target "photoshop"


/******************************************************************************************************
 * ICCプロファイル指定、自動クロップ　Photoshop用スクリプト (Ver.0.8)
 * できること：
 *	画像に対してICCプロファイルを指定する
 *	黒枠を自動で除去する
 * テストはWindows版 Photoshop CS4 (64bit版)で行っています。
 *
 * Copyright (C) 2011 tsubu@coffee shop
 ******************************************************************************************************/
myconfig_target_project = "";

(function(){
// $.locale = "en";		// for I18n tests
$.localized = true;

// path of the ICC Profile
//	Win7 : C:\Windows\System32\spool\drivers\color
const ColorICCProfiles = ["None", "sRGB IEC61966-2.1",
								"fi-6140-sRGB.icm",
								"ES-H300_raw.icm",
								"CaminacsW-NS-CS2.icm" ];
const GrayICCProfiles = ["None", "Gray Gamma 2.2", 
								"sGray", 
								"EPSON Gray - Gamma 2.2", "EPSON Gray - Gamma 1.8" ];

const ClosedEdge = ["None", 
					localize( {en : "right binding",	ja : "右閉じ"}), 
					localize( {en : "left binding",		ja : "左閉じ"}),
					localize( {en : "top binding",		ja : "上閉じ"}),
					localize( {en : "bottom binding",	ja : "下閉じ"}) ];

const BookStyle = ["None", 
					localize( {en : "105x148(mm) : ISO A6", ja : "105x148(mm) : A6(文庫)"} ), 
					localize( {en : "103x182 : B40",		ja : "103x182 : B40(新書判)"} ), 
					localize( {en : "128x182 : JIS B6",	ja : "128x182 : B6(単行本)"} ),
					localize( {en : "148x210 : ISO A5",	ja : "148x210 : A5(大判コミック)"} ), 
					localize( {en : "182x232 : JIS B5variable",	ja : "184x232 : B5変形(技術書等)"} ),
					localize( {en : "182x257 : JIS B5",	ja : "182x257 : B5"} ),
					localize( {en : "210x284 : ISO A4variable",	ja : "210x284 : A4変形(雑誌)"} ),
					localize( {en : "210x297 : ISO A4",	ja : "210x297 : A4"} )];

// 用紙サイズ (mm)。BookStyle の順にあわせるように。
const BookStyleSize = new Array( [220, 300],	// dummy
											[105, 148],		// A6 : 文庫本
											[103, 182],		// 新書判
											[128, 182],		// B6 : 単行本
											[148, 210],		// A5 : 大型コミック
											[182, 232],		// 一部技術書 (オライリーなど)
											[182, 257],		// B5: (小型) 雑誌、同人誌、学習用ノート
											[210, 284],		// 一部雑誌
											[210, 297]		 // A4: (大型) 雑誌
											);

const Myconfigs_ProjectFileName = "CS-PS4CropPicture-ProjectInfo.json";
const Myconfigs_binarizationThreshold = 22;			// defaultの2値化閾値
const Myconfigs_binarization2ndRatio = 0.2;			// 黒枠の許容率

const ScriptName_cropPicture = "CS-PS4CropPicture_process.jsx";
var ListOfAutoCropProjects = new Array();

// ======================================================================
function writeJsonFile(file, json) {
		file.encoding = "UTF8";
//		file.lineFeed = "unix";

		file.open("w", "TEXT", "????");
		// unicode signature, this is UTF16 but will convert to UTF8 "EF BB BF"
//		file.write("\uFEFF");
		file.write(json);
		
		file.close();
}


/**********
 * for GUI
 **********/
 function Params() {
	this.book_size = 0;
	this.closed_edge = 0;

	this.icc_color = 0;
	this.icc_gray = 0;
	this.save16bits = true;
	this.saveTiff = false;

	this.do_auto_crop = true;
		this.autoCalc_binarization_threshold = false;
		this.binarization_threshold = Myconfigs_binarizationThreshold;
		this.binarization_backgroundRatio = Myconfigs_binarization2ndRatio;
		this.hCropPadding = 0;
		this.vCropPadding = 0;

	this.do_LensCorrection = false;
		this.LensCorrection_brightness = 4.00;
		this.LensCorrection_center = 50;
}


function MyConfigs() {
	this.params = new Params();
}


MyConfigs.prototype = {

	get_icc_name :  function(target) {
		if (target == "color") {
			return ColorICCProfiles[this.params.icc_color];
		}
		return GrayICCProfiles[this.params.icc_gray];
	},

	get_closedEdge : function() {
		return ClosedEdge[ this.params.closed_edge ];
	},
/*
	get_paper_pixcels :  function(dpi) {
		return paper_pixels(this.params.book_size, dpi);
	},
*/
}

MyConfigs.prototype.WriteJson = function(path) {
	var json = uneval(this.params);
	var fileObj = new File(path + "/" + Myconfigs_ProjectFileName);
	writeJsonFile(fileObj, json);
	// alert(json);

	for (var i in ListOfAutoCropProjects) {
		if (ListOfAutoCropProjects.hasOwnProperty(i) && ListOfAutoCropProjects[i] == path)	// 既にある
			return;
	}
	ListOfAutoCropProjects.push(path);
}


MyConfigs.prototype.set_entity = function(uDlg) {
	// ユーザ設定の保存
	with (uDlg.menus.g1.ICC) {
		if (Color.list.selection) {
			this.params.icc_color = parseInt( Color.list.selection );		// parseIntがなければ "A5" とかの値だった
		}
		this.params.icc_color = this.get_icc_name("color");
		if (Gray.list.selection) {
			this.params.icc_gray = parseInt( Gray.list.selection );
		}
		this.params.icc_gray = this.get_icc_name("gray");
	}

	with (uDlg.menus.g1.bookInfo) {
		if (BookSize.list.selection) {
			this.params.book_size = parseInt( BookSize.list.selection );
		}

		if (ClosedEdge.list.selection) {
			this.params.closed_edge = parseInt( ClosedEdge.list.selection );
		}
		this.params.closed_edge = this.get_closedEdge();
	}

	with (uDlg.menus.g2.autoCrop) {
		this.params.do_auto_crop = chk.value;
		this.params.autoCalc_binarization_threshold = chkAutoCalc.value;
		this.params.binarization_threshold = binarization.slider.value;
		this.params.binarization_backgroundRatio = backgroundRatio.slider.value;
		this.params.hCropPadding = hPadding.slider.value;
		this.params.vCropPadding = vPadding.slider.value;
	}

	with (uDlg.menus.g2.Save) {
		this.params.save16bits = chk_save16bits.value;
		this.params.saveTiff = chk_saveTiff.value;
	}

	with (uDlg.menus.g2.LensCorrection) {
		this.params.do_LensCorrection = chk.value;
		this.params.LensCorrection_brightness = PIC_brightness.slider.value;
		this.params.LensCorrection_midpoint = PIC_midpoint.slider.value;
	}
}


MyConfigs.prototype.set_params_byGUI = function() {
	var that = this;		// avoid for this.parent.parent.... I think  it is ugly.

	var res = """dialog { orientation: 'column', alignChildren: 'fill',
		description: StaticText { text : 'CS-PS4CropPicture ver. 0.7.0' },

		menus : Group { orientation: 'row',
			g1 : Group { orientation: 'column', alignChildren: 'fill', alignment:['fill','top'],
				bookInfo : Panel { orientation: 'row', alignChildren: 'top',
					text : 'book info',
					BookSize : Group { orientation : 'column', alignChildren: 'left',
						st : StaticText { text : 'book size' },
						list : ListBox { },
					},
					ClosedEdge : Group { orientation: 'column', alignChildren: 'left',
						st : StaticText { text : 'closed direction :' },
						list : ListBox { },
					},
				},
				ICC : Panel { orientation: 'row', alignChildren: 'top',
					text : 'ICC info',
					Color : Group { orientation : 'column', alignChildren: 'left',
						st : StaticText { text : 'specify color profile :' }
						list : ListBox {  },
					},
					Gray : Group { orientation : 'column', alignChildren: 'left',
						st : StaticText { text : 'specify gray profile :' }
						list : ListBox {  },
					},
				},
			},
		
			g2 : Group { orientation: 'column', alignChildren: 'fill',
				autoCrop : Panel { alignChildren : 'left',
					text: 'Auto Crop',
					chk : Checkbox { text : 'do automatic crop', value : true },
					chkAutoCalc : Checkbox { text : 'auto calc the binarization threshold (experiment)', value : false },
					binarization : Group { orientation: 'row',
						st : StaticText { text : 'binalization threshold [1,254] :  ' },
						et : EditText { characters:4 },
						slider : Slider { minvalue : 1, maxvalue : 254 }
					},
					backgroundRatio : Group {
						st : StaticText { text : 'background ratio [0,1] :  ' },
						et : EditText { characters:4 },
						slider : Slider { minvalue : 0, maxvalue : 1 }
					},
					hPadding : Group {
						st : StaticText { text : 'horizontal padding [-100, 100] :' },
						et : EditText { characters:4 },
						slider : Slider { minvalue : -100, maxvalue : 100 }
					},
					vPadding : Group {
						st : StaticText { text : 'vertical padding [-100, 100] :' },
						et : EditText { characters:4 },
						slider : Slider { minvalue : -100, maxvalue : 100 }
					},
				},
				Save : Panel { orientation: 'row',
					text : 'save',
					chk_save16bits : Checkbox { text : 'save 16bit / channel', value : true },
					chk_saveTiff : Checkbox { text : 'save to TIFF', value : false },
				},

				LensCorrection : Panel { alignChildren : 'left',
					text : 'lighting to the edges of the image by Lens Correction (heavy process)',
					chk : Checkbox { text : 'execute', value : false },
					PIC_brightness : Group { orientation: 'row',
						st : StaticText { text : 'amount [-100,100] : ' },
						et : EditText { characters : 4 },
						slider : Slider { minvalue : -100, maxvalue : 100 }
					},
					PIC_midpoint : Group { orientation: 'row',
						st : StaticText { text : 'midpoint [0,100] : ' },
						et : EditText { characters : 3 },
						slider : Slider { minvalue : 0, maxvalue : 100 }
					},
				},
			},
		},

		buttoms : Group { orientation : 'row', alignment : 'right',
			createPrjBtn : Button { text : 'Project folder' },
			okBtn : Button { text : 'OK', properties: {name : 'ok'} }
			cancelBtn : Button { text : 'Cancel', properties:{name:’cancel’} }
		}
	}""";
	var uDlg = new Window(res);

	var ok_or_cancel = true;
	with (uDlg.buttoms) {
		cancelBtn.onClick = function() {
			ok_or_cancel = false;
			uDlg.close();			// ハンドラ登録すると消えなくなるので必要
		}

		createPrjBtn.text = localize( {en : 'Project folder', ja : '適用フォルダ'} );
		// この中のthisはボタンオブジェクトになっている。this.parentはGroup。なのでthatを使用して上の関数を呼び出し。
		createPrjBtn.onClick = function () {
			var project_folder = Folder.selectDialog("crop対象のフォルダを指定して下さい。");		// select top folder
			that.set_entity(uDlg);
			// set_entity(uDlg);

			that.WriteJson(project_folder.fullName);
		}
	}

	with (uDlg.menus.g1) {
		bookInfo.BookSize.st.text = localize( {en : 'book size :', ja : '判型 :' } );
		bookInfo.ClosedEdge.st.text = localize({en : 'closed direction :', ja : '本を閉じている方向 :' } );
		ICC.Color.st.text = localize( { en : 'specify color profile :', ja : 'カラー原稿への指定プロファイル :' } );
		ICC.Gray.st.text = localize( { en : 'specify gray profile :', ja : 'グレー原稿への指定プロファイル :' } );

		var i;
		for ( i in BookStyle) {
			bookInfo.BookSize.list.add( 'item', BookStyle[i] );
		}
		for ( i in ClosedEdge) {
			bookInfo.ClosedEdge.list.add( 'item', ClosedEdge[i] );
		}
		for ( i in ColorICCProfiles) {
			ICC.Color.list.add( 'item', ColorICCProfiles[i] );
		}
		for ( i in GrayICCProfiles) {
			ICC.Gray.list.add( 'item', GrayICCProfiles[i] );
		}
	}

	with (uDlg.menus.g2.autoCrop) {
		// 2階調化の閾値
		binarization.st.text = localize( {en : 'binalization threshold [1, 254] :', ja : '2階調化 閾値 [1,254] :'} );
		binarization.slider.size = 'width : 253, height : 20';
		binarization.slider.onChanging = function() {
			binarization.et.text =  Math.floor(this.value);
		}
		binarization.et.onChange = function() {
			binarization.slider.value = Number(this.text);
		}
		binarization.slider.value = this.params.binarization_threshold;
		binarization.et.text =  Math.floor(this.params.binarization_threshold);

		// 黒の割合
		backgroundRatio.st.text = localize( {en : 'background ratio [0, 1] :', ja : '背景の許容度 [0,1] :'} );
		backgroundRatio.slider.onChanging = function() {
			backgroundRatio.et.text =  this.value;
		}
		backgroundRatio.et.onChange = function() {
			backgroundRatio.slider.value = Number(this.text);
		}
		backgroundRatio.slider.value = this.params.binarization_backgroundRatio;
		backgroundRatio.et.text =  this.params.binarization_backgroundRatio;

		// 水平方向オフセット
		hPadding.st.text = localize( {en : 'horizontal padding [-100, 100] :', ja : '左右へのpadding [-100,100] :'} );
		hPadding.slider.size = 'width : 200, height : 20';
		hPadding.slider.onChanging = function() {
			hPadding.et.text =  Math.floor(this.value);
		}
		hPadding.et.onChange = function() {
			hPadding.slider.value = Number(this.text);
		}
		hPadding.slider.value = this.params.hCropPadding;
		hPadding.et.text =  Math.floor(this.params.hCropPadding);

		// 垂直方向オフセット
		vPadding.st.text = localize( {en : 'vertical padding [-100, 100] :', ja : '上下へのpadding [-100,100] :'} );
		vPadding.slider.size = 'width : 200, height : 20';
		vPadding.slider.onChanging = function() {
			vPadding.et.text =  Math.floor(this.value);
		}
		vPadding.et.onChange = function() {
			vPadding.slider.value = Number(this.text);
		}
		vPadding.slider.value = this.params.vCropPadding;
		vPadding.et.text =  Math.floor(this.params.vCropPadding);
	}

	with (uDlg.menus.g2.LensCorrection) {		//  PIC : PeripheralIlluminationCorrection
		text = localize( {en : 'lighting to the edges of the image by Lens Correction (heavy process)',
								ja : 'レンズ補正(周辺光量補正)で端を持ち上げる (重い処理)'} );
		chk.text = localize( {en : 'execute', ja : '実行する'} );

		PIC_brightness.st.text = localize( {en : 'amount [-100,100] :', ja : '適用量 [-100,100] :'} );
		PIC_brightness.slider.size = 'width : 200, height : 20';
		PIC_brightness.slider.onChanging = function() {
			PIC_brightness.et.text = Math.floor(this.value);
		}
		PIC_brightness.et.onChange = function() {
			PIC_brightness.slider.value = Number(this.text);
		}
		PIC_brightness.et.text = this.params.LensCorrection_brightness;		// init

		PIC_midpoint.st.text = localize( {en : 'midpoint [0,100] :', ja : '中心点 [0,100] :'} );
		PIC_midpoint.slider.size = 'width : 100, height : 20';
		PIC_midpoint.slider.onChanging = function() {
			PIC_midpoint.et.text = Math.floor(this.value);
		}
		PIC_midpoint.et.onChange = function() {
			PIC_midpoint.slider.value = Number(this.text);
		}
		PIC_midpoint.et.text = this.params.LensCorrection_center;		// init
	}

	/* -------- main -------- */
	uDlg.show();

	return ok_or_cancel;
}



function MyProjects() { }

MyProjects.prototype.run = function() {
	var res = """dialog { alignChildren: 'fill',
			list : ListBox {  },
			buttoms : Group { orientation : 'row', alignment : 'right',
				addPrjBtn : Button { text : 'プロジェクトの追加' },
				deletePrjBtn : Button { text : '登録プロジェクトの削除' },
				okBtn : Button { text : 'バッチ処理開始', properties: {name : 'ok'} }
				cancelBtn : Button { text : '中止する', properties:{name:’cancel’} }
			},
			}""";
	var uDlg = new Window(res);
	with (uDlg) {
		var i;
		for ( i in ListOfAutoCropProjects) {
			list.add( 'item', ListOfAutoCropProjects[i] );
		}
	}

	var ok_or_cancel = true;
	with (uDlg.buttoms) {
		addPrjBtn.text = localize( {en : 'add Project', ja : 'プロジェクトの追加'} );
		addPrjBtn.onClick = function() {
			var project_folder = Folder.selectDialog( localize( {en : 'specify the folder that has crop targets',
																				ja : "crop対象のフォルダを指定して下さい。"} ));		// select top folder
			if (uDlg.list.find(project_folder.fullName) == null)
				uDlg.list.add( 'item', project_folder.fullName );
		}

		deletePrjBtn.text = localize( {en : 'delete the registerd project', ja : '登録プロジェクトの削除'} );
		deletePrjBtn.onClick = function() {
			var n = uDlg.list.selection.index;
			uDlg.list.remove(uDlg.list.items[n]);
		}

		okBtn.text =  localize( {en : 'Start batch', ja : 'バッチ処理開始'} );
		cancelBtn.text = localize( {en : 'Cancel', ja : '中止する'} );
		cancelBtn.onClick = function() {
			ok_or_cancel = false;
			uDlg.close();			// ハンドラ登録するとボタン押してもdaialogが消えなくなるのでcloseが必要
		}
	}

	uDlg.show();

	ListOfAutoCropProjects = uDlg.list.items;

	return ok_or_cancel;
}




// ======================================================================
// main
// ======================================================================
try {
	var myconfig = new MyConfigs();
	var process_ok = true;

	process_ok = myconfig.set_params_byGUI();
	if (! process_ok)
		throw new Error("途中でcancelされました");

	process_ok = new MyProjects().run();
	if (! process_ok)
		throw new Error("途中でcancelされました");

	/* batch processing */
	var autoCrop_batch_path = File($.fileName).parent.fullName + "/" + ScriptName_cropPicture;
	if (! File(autoCrop_batch_path).exists) {
		return;
	}

	for (var i = 0; i < ListOfAutoCropProjects.length; i++) {
		myconfig_target_project = ListOfAutoCropProjects[i];

		$.evalFile(autoCrop_batch_path);
		$.gc();
	}

} catch (e) {
	alert(e.message + "\nファイル : " + e.fileName + "\n行数 ： " + e.line);
}


})();

