﻿#script "CS-PS4CropPicture_process"
#target "photoshop"

/******************************************************************************************************
 * ICCプロファイル指定、自動クロップ　Photoshop用スクリプト (Ver.0.8)
 * できること：
 *	画像に対してICCプロファイルを指定する
 *	黒枠を自動で除去する
 * テストはWindows版 Photoshop CS4 (64bit版)で行っています。
 *	ScanTailorのプロジェクトファイルの読み込みでXMLを使用しているため CS3以降が必要になります。
 *
 * Copyright (C) 2011 tsubu@coffee shop
 ******************************************************************************************************/

(function() {
// $.locale = "en";		// for I18n tests
$.localized = true;

const ClosedEdge = ["None", "右閉じ", "左閉じ", "上閉じ", "下閉じ"];

// 用紙サイズ (mm)。BookStyle の順にあわせるように。
const BookStyleSize = new Array( [220, 300],
											[105, 148],		// A6 : 文庫本
											[103, 182],		// 新書判
											[128, 182],		// B6 : 単行本
											[148, 210],		// A5 : 大型コミック
											[182, 232],		// 一部技術書 (オライリーなど)
											[182, 257],		// B5: (小型) 雑誌、同人誌、学習用ノート
											[210, 284],		// 一部雑誌
											[210, 297]		 // A4: (大型) 雑誌
											);

const Myconfigs_ProjectFileName = "CS-PS4CropPicture-ProjectInfo.json";
const TiffCompression = "None"		// "None", "LZW"

// ======================================================================
function __does_file_exists(path) {
	if (! path)	return "";

	if (File(path).exists) {
		return path;
	}
	return "";
}

function does_file_exists(proj_dir, dir, fname) {
	var path;

	while (dir) {
		path = __does_file_exists(dir + "/" + fname);
		if (path)	return path;

		if (dir == proj_dir)	return "";

		var parent = Folder(dir).parent;
		if (parent) 
			dir = parent.fullName;
		else
			return "";
	}

	return "";
}

// ************************************************************
/**
 * 画像の回転設定と回転
 */
function RotateSetting(proj_path, fpath) {
	this.path;
	this.hash = new Array();
	
	this.check_file = function(proj_path, fpath) {
		// ScanTailor
		this.path = does_file_exists(proj_path, fpath, "st.ScanTailor");
		if (this.path){
			this.read_ScanTailorProject(this.path);
			return true;
		}

		// eTilTran
		this.path = does_file_exists(proj_path, fpath, "project.txt");
		if (this.path){
			this.read_eTilTranProject(this.path);
			return true;
		}

		this.path = "";
		return false;
	}
	this.check_file(proj_path, fpath);		// 初期化の一部
}
// ************
//  for ScanTailor
//
RotateSetting.prototype.read_ScanTailorProject = function(path) {
	try {
		var conf_file = new File(path);
		// conf_file.encoding = "UTF-8";
		var confXML = "";
		if (conf_file.open("r")){
			confXML += conf_file.read();
			if (confXML != "") {
				var rootXML = new XML(confXML);
				confXML = null;		// 巨大になるのですぐに廃棄
			};
			conf_file.close();
			
			this.getAngles(rootXML);
			rootXML = null;		// 巨大になるのですぐに廃棄
		}
	} catch (e) {
		this.path = "";
	}
}
	
// XMLから角度データをもってくる。ファイル名と角度のハッシュを保存
RotateSetting.prototype.getAngles = function(rootXML) {
	// var num = rootXML.filters.deskew.page.length();
	var npage = rootXML.pages.page.length();

	var fname;
	var file_id;
	var image_id;
	var page_id;
	var angle;
	for (var i = 0; i < npage; i++) {
		 page_id = rootXML.filters.deskew.page[i].@id;
		 angle = rootXML.filters.deskew.page[i].params.@angle;
		 
		 image_id = rootXML.pages.page.(@id == page_id).@imageId;
		 
		 file_id = rootXML.images.image.(@id == image_id).@fileId;
		 fname = rootXML.files.file.(@id == file_id).@name;
		 
		 this.hash[fname.toString()] = Number(angle.toString());
		 // this.hash[fname.toString()] = angle.toString();
	}

	return;
}

// **********
// for eTilTran
// **********
RotateSetting.prototype.read_eTilTranProject = function(path) {
	try {
		var num = 0;
		var line = "";
		var base_text_rect = "";

		var file = new File(path);
		file.open("r");
		while (!file.eof){
			line = file.readln();
			// alert ("line:" + line);
			if (line.match(/^BaseTextRect/)) {
				base_text_rect = line.split(",")[1];		// 2つ目の値
			}
			if (! line.match(/^Num/) ) {		// Num(ファイル数) がヘッダ領域の最後
				continue;
			}
			num = line.split(",")[1];
			break;
		}

		while (!file.eof) {
			line = file.readln();
			var tmp_values = line.split(",");

			// ファイル名 : 角度 (filename : angle)。反時計回りが正方向だった。
			this.hash[tmp_values[0]] = - this.trans_from_etiltran_angle_form(tmp_values[6]);
		}
		file.close();
	} catch (e) {
		alert(e);
		this.path = "";
	}
}

RotateSetting.prototype.trans_from_etiltran_angle_form = function(str_angle) {
	var tmp = 0.0;
	if (! isNaN(str_angle)) {
		tmp = parseFloat(str_angle);
	}
	return tmp;
}


// 回転処理をを行う
//	StanTailor  (v9.10.0) の GUI はプロジェクトの－値を表示している (方向が逆していた)。
//	プロジェクトでは時計回りが正方向となる。(== Photoshopと同じ)
RotateSetting.prototype.do_process = function(fname) {
	if (this.path == "")	return;

	if (fname in this.hash) {
		var angle = this.hash[fname];
		if (angle != 0)
			activeDocument.rotateCanvas(angle);
			logger.write( "\t(" + angle.toFixed(4) + ")" );
	}
}



// ************************************************************
/**
 * 画像のクロップ設定とクロップ処理関連
 */
function CropSetting (proj_path, path) {
	this.path = path;	// path of the config file for crop
	this.crop_info;		// JSON data

	this.do_AutoCrop = myconfig.params.do_auto_crop;
	this.autoCalcBinarizationThreshold = myconfig.params.autoCalc_binarization_threshold;
	this.binarizationThreshold = myconfig.params.binarization_threshold;
	this.binarization1stRatio = (myconfig.params.binarization_backgroundRatio < 0.6) ? 0.6 : myconfig.params.binarization_backgroundRatio;
	this.binarization2ndRatio = myconfig.params.binarization_backgroundRatio;

	this.prev_width = -1;
	this.prev_height = -1;
	this.checkThreshold = 80;

	this.check_file = function () {
		this.path = does_file_exists(proj_path, this.path, "cropInfo.txt");
		if (! this.path){
			return;
		}

		try {
			var conf_file = new File(this.path);
			// conf_file.encoding = "UTF-8";
			var confJSON = "";
			if (conf_file.open("r")){
				confJSON += conf_file.read();
				if (confJSON != "") {
					this.crop_info = eval( '('+confJSON  +')' );
				};
				conf_file.close();

				if (this.crop_info['mode'] == 'auto')
					this.do_AutoCrop = false;
			} else {
				this.crop_info = {};
			}			

		} catch (e) {
			this.path = "";
		}
	}
	this.check_file();
}

CropSetting.prototype.do_process = function ( doc_name ) {
	if (! this.path){
		if (this.do_AutoCrop)
			this.doAutoCrop( doc_name );
		return 0;
	}

	try {
		if (this.do_AutoCrop) {
			this.doAutoCrop( doc_name );
		} else {
			tmp_array = this.crop_info['area'];
			activeDocument.crop( tmp_array );
		}
	} catch(e) {
		alert(e.message + "\nファイル : " + e.fileName + "\n行数 ： " + e.line);		// 例外の発生
	}
	return 1;
}

CropSetting.prototype.doAutoCrop = function( doc_name )
{
	// 解析用のドキュメントを開く
	var tmp_doc_name =  "bi_"+ doc_name;
	activeDocument.duplicate(tmp_doc_name);
	
	var crop_region = this.detectPaperPosition(this.binarizationThreshold, this.binarization1stRatio, this.binarization2ndRatio);
	activeDocument.close(SaveOptions.DONOTSAVECHANGES);

	if (crop_region[2] - crop_region[0] < 4 || crop_region[3] - crop_region[1] < 4)	// ある程度の幅がないとcropしない
		return;

	activeDocument.crop(crop_region);
	
	// ログ用の処理
	if (this.prev_width < 0) {
		logger.write("\t[" + crop_region + "]\t[" + activeDocument.width.value + ", " +  activeDocument.height.value + "]FP");
		// logger.writeln(doc_name + "\t[" + crop_region + "]\t[" + activeDocument.width.value + ", " +  activeDocument.height.value + "]");
	} else if (Math.abs(this.prev_width - activeDocument.width.value) > this.checkThreshold ||
			      Math.abs(this.prev_height - activeDocument.height.value) > this.checkThreshold) {
		logger.write("\t[" + crop_region + "]\t[" + activeDocument.width.value + ", " +  activeDocument.height.value + "]TD");
	} else {
		logger.write("\t[" + crop_region + "]\t[" + activeDocument.width.value + ", " +  activeDocument.height.value + "]");
	}
	this.prev_width = activeDocument.width.value;
	this.prev_height =  activeDocument.height.value;
}

// 黒枠を取り除いた原稿位置を検出
CropSetting.prototype.detectPaperPosition = function(binarization_threshold, first_bg_ratio, second_bg_ratio)
{
	if (activeDocument.mode != DocumentMode.GRAYSCALE)
	{
		do_channelMixser_orengefilter();		// 紙やけの考慮
		convert_GrayScale_mode();			// カラーモードをグレーへ
	}
	var width = activeDocument.width.value;
	var height = activeDocument.height.value;

	if (this.autoCalcBinarizationThreshold)
		binarization_threshold = this.calcBinarizationThreshold(width, height);

	var color_reverse = (binarization_threshold > 128) ? true : false;

	// 2値化
	image_binarization(binarization_threshold);
	// change_binarization_mode(activeDocument.resolution);		// histogramがとれなくなるので使わない

	// *******************************
	// crop枠の検出、borderの認識
	// *******************************
	var first_bg_ratio_w = first_bg_ratio;
	var first_bg_ratio_h = first_bg_ratio;
	if (myconfig.params.book_size != 0) {
		var paperSize = myconfig.get_paper_pixcels(activeDocument.resolution, width, height);
		first_bg_ratio_w = paperSize[0] / width;
		first_bg_ratio_h = paperSize[1] / height;
	}

	var right, left, top, bottom;
	var steps = [64, 32, 16, 8, 4, 2];
	for (var j in steps) {
		var crr_step = steps[j];
		if (j == 0) {
			// 一時解析
			left = this.skipBlackBorder(true, 0, height,
												0, width / 2,			// 0, width / 4,
												crr_step, first_bg_ratio_h, color_reverse);
			right = this.skipBlackBorder(true, 0, height,
												width, width / 2,	// width, width * 3/ 4,
												- crr_step, first_bg_ratio_h, color_reverse);
			top = this.skipBlackBorder(false, 0, width,
												0, height / 2,		// 0, height / 4,
												crr_step, first_bg_ratio_w, color_reverse);
			bottom = this.skipBlackBorder(false, 0, width,
												height, height / 2,	// height, height * 3/ 4,
												- crr_step, first_bg_ratio_w, color_reverse);
			continue;
		}

		// 詳細な解析
		var prev_step = steps[j-1];

		left = this.skipBlackBorder(true, top, bottom,
											(left - prev_step > 0) ? left - prev_step : 0,
											(left + prev_step < width) ? left + prev_step : width,
											crr_step, second_bg_ratio, color_reverse);
		right = this.skipBlackBorder(true, top, bottom,
											(right + prev_step < width) ? right +prev_step : width,
											(right - prev_step > 0) ? right - prev_step : 0,
											- crr_step, second_bg_ratio, color_reverse);
		top = this.skipBlackBorder(false, left, right,
											(top - prev_step > 0) ? top - prev_step : 0,
											(top + prev_step < height) ? top + prev_step : height,
											crr_step, second_bg_ratio, color_reverse);
		bottom = this.skipBlackBorder(false, left, right,
											(bottom + prev_step < height) ? bottom + prev_step : height,
											(bottom - prev_step > 0) ? bottom - prev_step : 0,
											- crr_step, second_bg_ratio, color_reverse);
	}

	// マージン処理
	if (myconfig.params.vCropPadding) {
		left -= myconfig.params.vCropPadding;
		if (left < 0)	left = 0;

		right += myconfig.params.vCropPadding;
		if (right > width)	right = width;
	}
	if (myconfig.params.hCropPadding) {
		top -= myconfig.params.hCropPadding;
		if (top < 0)	top = 0;

		bottom += myconfig.params.hCropPadding;
		if (bottom > height)	bottom = height;
	}

	return [left, top, right, bottom];
}

/** 
 * 黒帯をスキップして紙端までたどり着く
 *	is_x : true (x軸方向のスキップ), false (y軸方向のスキップ)
 *	linse_start, line_end : 扱うラインの端
 *	start, end : stepで探す位置の最初と最後(あきらめる位置)
 *	step : ライン数。> 0 (軸上大きくなる方向へ), < 0 (軸上小さくなる方向へ)
 *	ratio : ボーダー(黒枠) の許容できる割合(0,1) 。小数で指定 (0.2とか)。0から1までの値を指定すること。
 */
CropSetting.prototype.skipBlackBorder = function (is_x, line_start, line_end,   start, end,    step, bg_ratio, color_reverse)
{
	var line_weight = line_end - line_start;
	var threshold = line_weight * Math.abs(step) * bg_ratio;
	var first_step_threshold = line_weight * Math.abs(step) * 0.1;

	var i, amount;
	for (i=start;  Math.abs(i - start) < Math.abs(end - start); i += step) {
		if (is_x)
			activeDocument.selection.select( [[i, line_start], [i+step, line_start], [i+step, line_end], [i, line_end]] );
		else
			activeDocument.selection.select( [[line_start, i], [line_end, i], [line_end, i+step], [line_start, i+step]] );

		// If color_reverse is true, background is white.
		//	note: histogram[0] is black, histogram[255] is white
		amount = (color_reverse == false) ? activeDocument.channels[0].histogram[0] : activeDocument.channels[0].histogram[255];
		if (i == start && amount < first_step_threshold) {
			break;
		}
		if (amount < threshold) {
			break;
		}
	}

	return i;
}

/**
 * P-タイル法のアレンジで黒背景と原稿を分離する閾値を算出する
 *	width : 画像の幅
 *	height: 画像の高さ
 *	return : 閾値をあらわす数値 [0, 255]
 */
CropSetting.prototype.calcBinarizationThreshold = function(width, height) {
	var paperSize = myconfig.get_paper_pixcels(activeDocument.resolution, width, height);
	var pixelThreshold = width * height -  paperSize[0] * paperSize[1];
	if (pixelThreshold < 0)	pixelThreshold = 0;

	var histo_data = activeDocument.channels[0].histogram;
	var npixels = 0;
	var ret = 0;
	var min = width * height;

	var index;
	for (index = 0; index < 255; index++) {
		npixels += histo_data[index];
		if (npixels < pixelThreshold)
			continue;
		
		if (histo_data[index+1] - histo_data[index] > 100 || npixels > paperSize[0] * paperSize[1] * 1.5) {
			ret = index;
			break;
		}
	}

	return ret;
}

// ************************************************************

/**
 * 表示しているドキュメントを全て閉じる。
 */
function cleanup_current_documents()
{
	var ndocs = documents.length;
	for (var i=0; i < ndocs; i++) {
		close_img_file();
	}
}

/*
function crop_and_test( master_title )
{
	var ret = 0;
	do_CropPhoto();

	ndocs = documents.length;			// == app.documents.length
	if ( ndocs < 3 ) {
		if ( myconfig.is_range(parseFloat(activeDocument.width)), parseFloat(activeDocument.height)) {
			// alert( activeDocument.height + " x " + activeDocument.width);
			do_TiffSave(output_folder.fullName + "/" + master_title );
			ret = 1;
		}
	}

	// 開いたドキュメントのクリア
	for (var j=1; j < ndocs; j++) {
		activeDocument = documents[1];
		close_img_file();
	}
	return ret;
}
*/

/**
 * ファイル名から奇数偶数を判定する。(拡張子前の一文字から判定)
 */
function is_oddPage(label)
{
	var n = 1;
	if ((n = label.lastIndexOf('.')) != -1) {
		var num = label.charAt(n-1);
		if ( parseInt(num) %2 == 0 ) {
			return 0;		// even (偶数)
		 } else {
			return 1;		// odd (奇数)
		}
	}
	return -1;			// not understand (判定不能)
}

/**
 * ファイルの処理
 * 付記：
 *	16bitモードへの変更はICCプロファイルの前に行う。レベル補正を行うとトーンジャンプが発生したから。
 *	レンズ補正がカラーでないとできないためICCプロファイルの指定前に行っている。
 */
function do_file(color_or_gray, crop_setting, rot_setting)
{
	var doc_name = activeDocument.name;
	logger.write(doc_name);

	rot_setting.do_process(doc_name);

	// 黒枠削除 (remove the border)
	if (myconfig.params.do_auto_crop)
		crop_setting.do_process(activeDocument.name);

	// 16bit モードへ変換
	if (myconfig.params.save16bits) {
		if (activeDocument.bitsPerChannel  != BitsPerChannelType.SIXTEEN)
			activeDocument.bitsPerChannel = BitsPerChannelType.SIXTEEN;
	}

	// レンズ補正 (lens correction)
	if (myconfig.params.do_LensCorrection)
		adaptLensCorrection(myconfig.params.LensCorrection_brightness, myconfig.params.LensCorrection_midpoint);		// 紙端の持ち上げ

	// 白黒化 (Black & White)
	if (color_or_gray == "gray" && activeDocument.mode != DocumentMode.GRAYSCALE) {
		do_channelMixser_orengefilter();		// 紙やけの処理
		convert_GrayScale_mode();			// カラーモードをグレースケールへ変換
	}

	// ICCプロファイルの指定 (specify the ICC profile)
	// プロファイルがない場合は無条件に、ある場合は同じでないか確認して、指定する。
	if ( myconfig.get_icc_name(color_or_gray) != "None") {
		if ( activeDocument.colorProfileType == ColorProfile.NONE || 
			 activeDocument.colorProfileName != myconfig.get_icc_name(color_or_gray) )
			assignProfile( myconfig.get_icc_name(color_or_gray) );		// 指定プロファイルの適用
	}

	logger.writeln("");
}

/**
 * 対象とする拡張子を持つ場合 true, そうでない場合 false
 * getFiles("*.tif", "*.png", "*.bmp", "*.jpg") では最初のパラメータしか拾えないので代替
 */
function hasTargetFileExtension(file) {
	var regex = /(\.tif|\.tiff|\.png|\.bmp|\.jpg|\.jpeg)$/i;
	if (regex.test(file.name))	return true;
	return false;
}

function do_files(proj_top, folder, output_folder, color_or_gray)
{
	var fpath = folder.fullName;
	var subFolder;
	var files;

	if ( Folder( fpath + "/out" ).exists ) {	// ScanTailor
		subFolder = Folder( fpath + "/out" );

		files =  subFolder.getFiles(hasTargetFileExtension);
		if (files.length > 0) {
			fpath = subFolder.fullName;
		} else {
			files =  folder.getFiles(hasTargetFileExtension);	// プロジェクトだけの場合
		}
	} else if (Folder( fpath + "/!変換データ" ).exists ) {	// eTilTran
		subFolder = Folder( fpath + "/!変換データ");

		files =  subFolder.getFiles(hasTargetFileExtension);
		if (files.length > 0) {
			fpath = subFolder.fullName;
		} else {
			files =  folder.getFiles(hasTargetFileExtension);	// プロジェクトだけの場合
		}
	} else {
		 files =  folder.getFiles(hasTargetFileExtension);
	}

	var crop_setting = new CropSetting(proj_top, fpath);
	var rot_setting = new RotateSetting(proj_top, fpath);
	var doc_name = "";

	// var files =  folder.getFiles("*.tif", "*.png", "*.bmp", "*.jpg");
	for (var i=0; i < files.length; i++) {
		// var ndocs = 1;
		open_img_file(files[i].fullName);
		doc_name = activeDocument.name;

		do_file(color_or_gray, crop_setting, rot_setting);

		SaveImageFile(output_folder.fullName, doc_name );
		close_img_file();
	}
}


/**
 * 処理フォルダにある画像をcrop処理する。
 * 対象フォルダは カラーの場合 "*c-*", グレースケールの場合 "*g-*" でその下に画像がある。001c-, 002g- とパート毎に分けていることを想定している。
 * 加工したデータは cropped フォルダへ保存する。
 */
function do_crop_folder(top_folder)
{
	var crop_folder_path = top_folder.fullName + "/cropped_" + top_folder.name;
	var output_topfolder = new Folder(crop_folder_path);
	if (! output_topfolder.exist) {
		output_topfolder.create();
	}

	var output_folder;

	// for (folder in Folder.getFiles()) だと folderが stringに変換されてしまうのでNGだった
	var folders = top_folder.getFiles("*c-*");
	for (var j=0; j < folders.length; j++) {
		if (folders[j].name == output_topfolder.name)
			continue;

		output_folder = new Folder(crop_folder_path+ "/" + folders[j].name);
		if (! output_folder.exist) {
			output_folder.create();
		}

		// alert(folders[j].name);
		do_files(top_folder.fullName, folders[j], output_folder, "color");
	}

	// for (folder in Folder.getFiles()) だと folderが stringに変換されてしまうのでNGだった
	var folders = top_folder.getFiles("*g-*");
	for (var j=0; j < folders.length; j++) {
		if (folders[j].name == output_topfolder.name)
			continue;

		output_folder = new Folder(crop_folder_path+ "/" + folders[j].name);
		if (! output_folder.exist) {
			output_folder.create();
		}

		do_files(top_folder.fullName, folders[j], output_folder, "gray");
	}

	folders = null;
	output_folder = null;
	output_topfolder = null;
}


/**********
 * for GUI
 **********/
function MyConfigs() {
	this.params;
}

MyConfigs.prototype = {
	get_icc_name : function(target) {
		if (target == "color") {
			return this.params.icc_color;
		}
		return this.params.icc_gray;
	},

	get_paper_pixcels :  function(dpi, width, height) {
		return guessPaperPixels(this.params.book_size, dpi, width, height);
	},

	ReadJson : function(path) {
		var conf_file = new File(path);

		// conf_file.encoding = "UTF-8";
		if (! conf_file.open("r")){
			return;
		}

		var json_string = conf_file.read();
		conf_file.close();

		this.params = eval( json_string );
	},
}

// ************************************************************

/**
 * 1inch == 2.54cm
 * length * dpi / 25.4 (dot / mm) == pixel
 */
function paper_pixels(paperStyle, dpi) {
	var x = y = 0;

	x = BookStyleSize[paperStyle][0];
	y = BookStyleSize[paperStyle][1];
	return [ Math.floor( x * dpi / 25.4 ), Math.floor( y * dpi /25.4) ];
}

function guessPaperPixels(paperStyle, dpi, width, height) {
	if (paperStyle != 0) {
		return paper_pixels(paperStyle, dpi);
	}

	var x = y = 0;
	var i;
	for ( i in BookStyleSize) {
		if (i == 0)	continue;	// ignore "None"

		x = Math.floor( BookStyleSize[i][0] * dpi / 25.4 );
		y = Math.floor( BookStyleSize[i][1] * dpi / 25.4 );

		if ( x > width || y > height) {
			i -= 1;
			break;
		}
	}

	x = Math.floor( BookStyleSize[i][0] * dpi / 25.4 );
	y = Math.floor( BookStyleSize[i][1] * dpi / 25.4 );
	return [x, y];
}


/************
 * logging
 ************/
function MyLogger(path, fname) {
	try {
		var output_folder = new Folder(path);
		if (! output_folder.exist) {
			output_folder.create();
		}
		
		this.fileObj = new File(path + "/" + fname);
		this.fileObj.open("w");
		this.fileObj.writeln("crop log :");
		this.fileObj.writeln("file \(angle)\t[crop_area]\t[width, height]");	// title
	} catch(e) {
		this.fileObj = null;
		alert("failed to make a logging file.");
	}

	this.write = function( strings ) {
		if (! this.fileObj)	return;
		this.fileObj.write(strings);
	}
	this.writeln = function( strings ) {
		if (! this.fileObj)	return;
		this.fileObj.writeln(strings);
	}
	this.close = function() {
		if (! this.fileObj)	return;
		this.fileObj.close();
	}
}

// =======================================================
function open_img_file(path) {
    var idOpn = charIDToTypeID( "Opn " );
    var desc1 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
    desc1.putPath( idnull, new File( path ) );
    executeAction( idOpn, desc1, DialogModes.NO );
}

// =======================================================
function close_img_file()
{
    var idCls = charIDToTypeID( "Cls " );
    var desc10 = new ActionDescriptor();
    var idSvng = charIDToTypeID( "Svng" );
    var idYsN = charIDToTypeID( "YsN " );
    var idN = charIDToTypeID( "N   " );
    desc10.putEnumerated( idSvng, idYsN, idN );
    executeAction( idCls, desc10, DialogModes.NO );
}

// =======================================================
function close_doc() {
	var idCls = charIDToTypeID( "Cls " );
	executeAction( idCls, undefined, DialogModes.NO );
}

// =======================================================
// 角度補正して切り抜き
// =======================================================
function do_CropPhoto() {
	var idCropPhotosAutozerozerozeroone = stringIDToTypeID( "CropPhotosAuto0001" );
	executeAction( idCropPhotosAutozerozerozeroone, undefined, DialogModes.NO );
}

// =======================================================
// チャンネルミキサーによるグレースケール化　（やけを考慮してオレンジフィルター）
// =======================================================
function do_channelMixser_orengefilter()
{
	var idChnM = charIDToTypeID( "ChnM" );
    var desc3 = new ActionDescriptor();
    var idpresetKind = stringIDToTypeID( "presetKind" );
    var idpresetKindType = stringIDToTypeID( "presetKindType" );
    var idpresetKindFactory = stringIDToTypeID( "presetKindFactory" );
    desc3.putEnumerated( idpresetKind, idpresetKindType, idpresetKindFactory );
    var idUsng = charIDToTypeID( "Usng" );
    // desc3.putPath( idUsng, new File( "C:\\Program Files\\Adobe\\Adobe Photoshop CS4\\Presets\\Channel Mixer\\Black & White with Orange Filter (RGB).cha" ) );
    desc3.putPath( idUsng, new File(photoshop_install_dir + "/Presets/Channel Mixer/Black & White with Orange Filter (RGB).cha" ) );
	executeAction( idChnM, desc3, DialogModes.NO );
}

// =======================================================
// カラープロファイルの指定 (INPUT Profile)
// =======================================================
function assignProfile(color_profile_name)
{
	if (color_profile_name == "None" || color_profile_name == "") {
		return;
	}
	var idassignProfile = stringIDToTypeID( "assignProfile" );
    var desc2 = new ActionDescriptor();
    var idnull = charIDToTypeID( "null" );
        var ref1 = new ActionReference();
        var idDcmn = charIDToTypeID( "Dcmn" );
        var idOrdn = charIDToTypeID( "Ordn" );
        var idTrgt = charIDToTypeID( "Trgt" );
        ref1.putEnumerated( idDcmn, idOrdn, idTrgt );
    desc2.putReference( idnull, ref1 );
    var idprofile = stringIDToTypeID( "profile" );
    desc2.putString( idprofile, color_profile_name );
	executeAction( idassignProfile, desc2, DialogModes.NO );
}

// =======================================================
function convert_GrayScale_mode()
{
	var idCnvM = charIDToTypeID( "CnvM" );
    var desc7 = new ActionDescriptor();
    var idT = charIDToTypeID( "T   " );
    var idGrys = charIDToTypeID( "Grys" );
    desc7.putClass( idT, idGrys );
	executeAction( idCnvM, desc7, DialogModes.NO );
}

// =======================================================
function image_binarization(threshold)
{
	// 2値化
	var idThrs = charIDToTypeID( "Thrs" );
    var desc4 = new ActionDescriptor();
    var idLvl = charIDToTypeID( "Lvl " );
    desc4.putInteger( idLvl, threshold );
	executeAction( idThrs, desc4, DialogModes.NO );
}

// =======================================================
// モード変換。2値化
function change_binarization_mode(resolution)
{
	var idCnvM = charIDToTypeID( "CnvM" );
    var desc5 = new ActionDescriptor();
    var idT = charIDToTypeID( "T   " );
        var desc6 = new ActionDescriptor();
        var idRslt = charIDToTypeID( "Rslt" );
        var idRsl = charIDToTypeID( "#Rsl" );
        desc6.putUnitDouble( idRslt, idRsl, resolution );			// dpi
        var idMthd = charIDToTypeID( "Mthd" );
        var idMthd = charIDToTypeID( "Mthd" );
        var idDfnD = charIDToTypeID( "DfnD" );
        desc6.putEnumerated( idMthd, idMthd, idDfnD );
    var idBtmM = charIDToTypeID( "BtmM" );
    desc5.putObject( idT, idBtmM, desc6 );
	executeAction( idCnvM, desc5, DialogModes.NO );
}


// =======================================================
// フィルター→変形→レンズ補正  (Peripheral illumination correction)
//	適用量 [-100, 100] : <0 暗くなる、>0 明るくなる
//	中心点 [0, 100]
function adaptLensCorrection(brightness, midpoint)
{
	if (brightness < -100)	brightness = -100;
	else if (brightness > 100)	brightness = 100;

	if (midpoint< 0)	midpoint = 0;
	else if (midpoint > 100)	midpoint = 100;

	var idLnCr = charIDToTypeID( "LnCr" );
    var desc2 = new ActionDescriptor();
    var idLnIa = charIDToTypeID( "LnIa" );
    desc2.putDouble( idLnIa, 0.000000 );
    var idLnIzero = charIDToTypeID( "LnI0" );
    desc2.putDouble( idLnIzero, 0.000000 );
    var idLnIone = charIDToTypeID( "LnI1" );
    desc2.putDouble( idLnIone, 0.000000 );
    var idLnItwo = charIDToTypeID( "LnI2" );
    desc2.putDouble( idLnItwo, 0.000000 );
    var idLnIthree = charIDToTypeID( "LnI3" );
    desc2.putDouble( idLnIthree, 1.000000 );
    var idLnRa = charIDToTypeID( "LnRa" );
    desc2.putDouble( idLnRa, 0.000000 );
    var idLnVp = charIDToTypeID( "LnVp" );
    desc2.putDouble( idLnVp, 0.000000 );
    var idLnHp = charIDToTypeID( "LnHp" );
    desc2.putDouble( idLnHp, 0.000000 );
    var idLnSi = charIDToTypeID( "LnSi" );
    desc2.putDouble( idLnSi, 100.000000 );
    var idLnFt = charIDToTypeID( "LnFt" );
    desc2.putInteger( idLnFt, 2 );
    var idLnSb = charIDToTypeID( "LnSb" );
    desc2.putDouble( idLnSb, brightness );			// 周辺光量補正、適用量 (amount)
    var idLnSt = charIDToTypeID( "LnSt" );
    desc2.putInteger( idLnSt, midpoint );				// 周辺光量補正、中心点 (midpoint)
    var idLnRc = charIDToTypeID( "LnRc" );
    desc2.putInteger( idLnRc, 0 );
    var idLnBy = charIDToTypeID( "LnBy" );
    desc2.putInteger( idLnBy, 0 );
    var idLnNa = charIDToTypeID( "LnNa" );
    desc2.putInteger( idLnNa, 16 );
    var idLnIh = charIDToTypeID( "LnIh" );
    desc2.putInteger( idLnIh, 0 );
    var idLnIv = charIDToTypeID( "LnIv" );
    desc2.putInteger( idLnIv, 0 );
    var idLnIs = charIDToTypeID( "LnIs" );
        var desc3 = new ActionDescriptor();
        var idRd = charIDToTypeID( "Rd  " );
        desc3.putDouble( idRd, 127.000000 );
        var idGrn = charIDToTypeID( "Grn " );
        desc3.putDouble( idGrn, 127.000000 );
        var idBl = charIDToTypeID( "Bl  " );
        desc3.putDouble( idBl, 127.000000 );
    var idRGBC = charIDToTypeID( "RGBC" );
    desc2.putObject( idLnIs, idRGBC, desc3 );
    var idLnNm = charIDToTypeID( "LnNm" );
    desc2.putBoolean( idLnNm, true );
	executeAction( idLnCr, desc2, DialogModes.NO );
}


// =======================================================
function do_TiffSave(fpath, compress) {
	var idsave = charIDToTypeID( "save" );
    var desc2 = new ActionDescriptor();
    var idAs = charIDToTypeID( "As  " );
        var desc3 = new ActionDescriptor();
        var idBytO = charIDToTypeID( "BytO" );
        var idPltf = charIDToTypeID( "Pltf" );
        var idIBMP = charIDToTypeID( "IBMP" );
        desc3.putEnumerated( idBytO, idPltf, idIBMP );
		if (compress == "LZW") {
			var idLZWC = charIDToTypeID( "LZWC" );
			desc3.putBoolean( idLZWC, true );
		}
    var idTIFF = charIDToTypeID( "TIFF" );
    desc2.putObject( idAs, idTIFF, desc3 );
    var idIn = charIDToTypeID( "In  " );
    desc2.putPath( idIn, new File( fpath ) );
	executeAction( idsave, desc2, DialogModes.NO );
}

function do_BmpSave(fpath) {
	var idsave = charIDToTypeID( "save" );
    var desc2 = new ActionDescriptor();
    var idAs = charIDToTypeID( "As  " );
        var desc3 = new ActionDescriptor();
        var idPltf = charIDToTypeID( "Pltf" );
        var idPltf = charIDToTypeID( "Pltf" );
        var idWin = charIDToTypeID( "Win " );
        desc3.putEnumerated( idPltf, idPltf, idWin );
        var idBtDp = charIDToTypeID( "BtDp" );
        var idBtDp = charIDToTypeID( "BtDp" );
        var idBDtwofour = charIDToTypeID( "BD24" );
        desc3.putEnumerated( idBtDp, idBtDp, idBDtwofour );
        var idCmpr = charIDToTypeID( "Cmpr" );
        desc3.putBoolean( idCmpr, false );
    var idBMPF = charIDToTypeID( "BMPF" );
    desc2.putObject( idAs, idBMPF, desc3 );
    var idIn = charIDToTypeID( "In  " );
    desc2.putPath( idIn, new File(fpath) );
	executeAction( idsave, desc2, DialogModes.NO );
}

// インタレースなし
function do_PngSave(fpath ) {
	var idsave = charIDToTypeID( "save" );
    var desc4 = new ActionDescriptor();
    var idAs = charIDToTypeID( "As  " );
        var desc5 = new ActionDescriptor();
        var idPGIT = charIDToTypeID( "PGIT" );
        var idPGIT = charIDToTypeID( "PGIT" );
        var idPGIN = charIDToTypeID( "PGIN" );
        desc5.putEnumerated( idPGIT, idPGIT, idPGIN );
        var idPNGf = charIDToTypeID( "PNGf" );
        var idPNGf = charIDToTypeID( "PNGf" );
        var idPGAd = charIDToTypeID( "PGAd" );
        desc5.putEnumerated( idPNGf, idPNGf, idPGAd );
    var idPNGF = charIDToTypeID( "PNGF" );
    desc4.putObject( idAs, idPNGF, desc5 );
    var idIn = charIDToTypeID( "In  " );
    desc4.putPath( idIn, new File(fpath) );
	executeAction( idsave, desc4, DialogModes.NO );
}

// 標準8
function do_JpegSave( fpath ) {
	var idsave = charIDToTypeID( "save" );
    var desc6 = new ActionDescriptor();
    var idAs = charIDToTypeID( "As  " );
        var desc7 = new ActionDescriptor();
        var idEQlt = charIDToTypeID( "EQlt" );
        desc7.putInteger( idEQlt, 8 );
        var idMttC = charIDToTypeID( "MttC" );
        var idMttC = charIDToTypeID( "MttC" );
        var idNone = charIDToTypeID( "None" );
        desc7.putEnumerated( idMttC, idMttC, idNone );
    var idJPEG = charIDToTypeID( "JPEG" );
    desc6.putObject( idAs, idJPEG, desc7 );
    var idIn = charIDToTypeID( "In  " );
    desc6.putPath( idIn, new File(fpath) );
	executeAction( idsave, desc6, DialogModes.NO );
}

function SaveImageFile(folder_name, doc_name) {
	var save_name = folder_name+ "/" + doc_name;

	// TIFF保存指定の場合は TIFFで保存。
	if (myconfig.params.saveTiff) {
		do_TiffSave(save_name, TiffCompression);
		return;
	}

	// そうでない場合は元のフォーマットに出来るだけあわせる
	if (doc_name.match(/\.bmp/i)) {
		if (myconfig.params.save16bits) {
			if (activeDocument.bitsPerChannel  == BitsPerChannelType.SIXTEEN)
				activeDocument.bitsPerChannel = BitsPerChannelType.EIGHT;
		}
		do_BmpSave(save_name);
		return;
	}
	if (doc_name.match(/\.png/i)) {
		do_PngSave(save_name);
		return;
	}

	do_TiffSave(save_name, TiffCompression);
}


// ======================================================================
// main
// ======================================================================
// Save the current preferences
var startRulerUnits		= app.preferences.rulerUnits;
var startTypeUnits		= app.preferences.typeUnits;
var startDisplayDialogs	= app.displayDialogs;

var startBackgroundColor = app.backgroundColor;	// キャンバス回転のため一時的に背景を黒にする
app.backgroundColor.rgb.red = 0;
app.backgroundColor.rgb.green = 0;
app.backgroundColor.rgb.blue = 0;

var photoshop_install_dir = app.path.fullName;		// Photoshopをインストールしたフォルダ (x86 と 64版 では異なるためプリセットなどで問題が生じる)

// Set Adobe Photoshop CS4 to use pixels and display no dialogs
// app.preferences.rulerUnits = Units.CM;
app.preferences.rulerUnits = Units.PIXELS;
app.preferences.typeUnits = TypeUnits.PIXELS;
// app.displayDialogs = DialogModes.NO;


try {
	var top_folder;
	if (typeof myconfig_target_project == "undefined")
		top_folder = Folder.selectDialog( localize( {en : "specify the folder that has this script's project file",
																ja : "本スクリプト用のプロジェクトファイルがあるフォルダを指定して下さい。"}) );		// select top folder
	else
		top_folder = new Folder(myconfig_target_project);
	if (top_folder == null){
		// throw "no_input_folder";
		throw new Error( localize( {en : "don't specify the project folder", ja : "対象フォルダが指定されていません"}) );
	}

	var myconfig = new MyConfigs();
	myconfig.ReadJson(top_folder.fullName + "/" + Myconfigs_ProjectFileName);
	var logger = new MyLogger(top_folder.fullName+ "/cropped_" + top_folder.name, "00crop_log.txt");

	cleanup_current_documents();
	do_crop_folder (top_folder);

	// cleaning resource
	logger.close();
	top_folder = null;

} catch (e) {
	alert(e.message + "\nファイル : " + e.fileName + "\n行数 ： " + e.line);
}


// reverse the application preferences
app.preferences.rulerUnits = startRulerUnits;
app.preferences.typeUnits = startTypeUnits;
// app.displayDialogs = startDisplayDialogs;

app.backgroundColor = startBackgroundColor;

})();

